'use strict';

Object.defineProperty(exports, Symbol.toStringTag, { value: 'Module' });

const css = require('@vanilla-extract/css');
const store = require('../models/store.cjs');
const colors = require('../styles/tokens/colors.cjs');
const system_model = require('../models/system.model.cjs');
const platform = require('./platform.cjs');

const hexToRgb = (hex) => {
  const channels = hex.replace(
    /^#?([a-f\d])([a-f\d])([a-f\d])$/i,
    (m, r, g, b) => "#" + r + r + g + g + b + b
  ).substring(1).match(/.{2}/g).map((x) => parseInt(x, 16));
  return `rgb(${channels[0]}, ${channels[1]}, ${channels[2]})`;
};
const rgb = (color, alpha) => {
  const rgbColor = color.includes("rgb") ? color : hexToRgb(color);
  const partial = rgbColor.replace("rgb(", "").replace(")", "");
  return alpha ? `rgba(${partial}, ${alpha})` : `rgb(${partial})`;
};
const isDefaultAccent = (accent) => {
  return colors.DEFAULT_ACCENTS.includes(accent);
};
const getAccent = (accent, colorMode) => {
  return isDefaultAccent(accent) ? colors.accents[colorMode ?? "light"][accent] : accent;
};
const getAccentText = (colorMode) => {
  return colors.accentsForeground[colorMode ?? "light"];
};
const getAccentHover = (color) => {
  return rgb(color, "0.075");
};
const mediaQueryColorScheme = (mode) => `(prefers-color-scheme: ${mode})`;
const isValidThemeMode = (mode) => {
  return system_model.ModePreferences.includes(mode);
};
const resolveThemeMode = (defaultThemeMode) => {
  const hasHydrated = store.store.getState()._hasHydrated;
  if (platform.isSSR() || !hasHydrated) {
    return "light";
  }
  const savedTheme = store.store.getState().themeMode;
  if (isValidThemeMode(defaultThemeMode)) {
    if (defaultThemeMode !== savedTheme) {
      store.store.getState().setThemeMode(defaultThemeMode);
    }
    return defaultThemeMode;
  }
  if (isValidThemeMode(savedTheme)) {
    return savedTheme;
  }
  store.store.getState().setThemeMode("system");
  console.log("[resolveThemeMode] using system theme mode");
  return "system";
};
const isPreferLightMode = () => window.matchMedia && window.matchMedia("(prefers-color-scheme: light)").matches;
const isPreferDarkMode = () => window.matchMedia && window.matchMedia("(prefers-color-scheme: dark)").matches;
function childSelectors(styleVariantsSelector, selector, rule) {
  const allSelectors = Array.from(new Set(styleVariantsSelector.split(" ")));
  const targetSelector = allSelectors[allSelectors.length - 1];
  css.globalStyle(`${targetSelector} ${selector}`, rule);
}

exports.childSelectors = childSelectors;
exports.getAccent = getAccent;
exports.getAccentHover = getAccentHover;
exports.getAccentText = getAccentText;
exports.isDefaultAccent = isDefaultAccent;
exports.isPreferDarkMode = isPreferDarkMode;
exports.isPreferLightMode = isPreferLightMode;
exports.mediaQueryColorScheme = mediaQueryColorScheme;
exports.resolveThemeMode = resolveThemeMode;
exports.rgb = rgb;
