import { setElementVars, assignInlineVars } from '@vanilla-extract/dynamic';
import { merge } from 'lodash';
import { commonVars, themeVars, lightThemeClass, darkThemeClass } from '../themes.css.js';
import { buttonOverrides } from '../../ui/button/button.helper.js';
import { clipboardCopyTextOverrides } from '../../ui/clipboard-copy-text/clipboard-copy-text.helper.js';
import { connectModalOverrides } from '../../ui/connect-modal/connect-modal.helper.js';
import { connectModalHeadTitleOverrides } from '../../ui/connect-modal-head/connect-modal-head.helper.js';
import { installButtonOverrides } from '../../ui/connect-modal-install-button/connect-modal-install-button.helper.js';
import { connectQRCodeOverrides, connectQRCodeShadowOverrides } from '../../ui/connect-modal-qrcode/connect-modal-qrcode.helper.js';
import { connectModalQRCodeErrorOverrides, connectModalQRCodeErrorButtonOverrides } from '../../ui/connect-modal-qrcode-error/connect-modal-qrcode-error.helper.js';
import { connectModalQRCodeSkeletonOverrides } from '../../ui/connect-modal-qrcode-skeleton/connect-modal-qrcode-skeleton.helper.js';
import { buttonOverrides as buttonOverrides$1, buttonLabelOverrides, buttonSublogoOverrides } from '../../ui/connect-modal-wallet-button/connect-modal-wallet-button.helper.js';

const overrideSchemas = {
  button: buttonOverrides,
  "clipboard-copy-text": clipboardCopyTextOverrides,
  "connect-modal": connectModalOverrides,
  "connect-modal-head-title": connectModalHeadTitleOverrides,
  "connect-modal-install-button": installButtonOverrides,
  "connect-modal-qr-code": connectQRCodeOverrides,
  "connect-modal-qr-code-shadow": connectQRCodeShadowOverrides,
  "connect-modal-qr-code-error": connectModalQRCodeErrorOverrides,
  "connect-modal-qr-code-error-button": connectModalQRCodeErrorButtonOverrides,
  "connect-modal-qr-code-loading": connectModalQRCodeSkeletonOverrides,
  "connect-modal-wallet-button": buttonOverrides$1,
  "connect-modal-wallet-button-label": buttonLabelOverrides,
  "connect-modal-wallet-button-sublogo": buttonSublogoOverrides
};
class OverrideStyleManager {
  _overrideMap;
  _theme;
  constructor(theme) {
    this._overrideMap = {};
    this._theme = theme;
  }
  update(overrideMap, theme) {
    if (overrideMap) {
      this._overrideMap = overrideMap;
    }
    if (theme) {
      this._theme = theme;
    }
  }
  applyOverrides(component) {
    const schema = overrideSchemas[component];
    const componentOverrideValue = this._overrideMap[component] ?? {};
    const configByTheme = groupByTheme(componentOverrideValue);
    if (!schema || !configByTheme) {
      return {};
    }
    const config = configByTheme[this._theme];
    const varsMap = {};
    for (const [vanillaCssVar, propertyName] of schema.overrides) {
      const propertyValue = config[propertyName];
      if (!propertyValue) {
        continue;
      }
      varsMap[`${vanillaCssVar}`] = propertyValue;
    }
    return assignInlineVars(varsMap);
  }
}
function groupByTheme(config) {
  if (!config) return null;
  const result = {
    light: {},
    dark: {}
  };
  for (const [overrideProperty, propertyConfig] of Object.entries(config)) {
    for (const theme of ["light", "dark"]) {
      result[theme] = {
        ...result[theme],
        [overrideProperty]: propertyConfig[theme]
      };
    }
  }
  return result;
}
function assignThemeVars(customTheme, colorMode) {
  const schemeClass = colorMode === "light" ? lightThemeClass : darkThemeClass;
  const elements = document.getElementsByClassName(schemeClass);
  const mergedVars = merge(commonVars, customTheme);
  for (let el of elements) {
    setElementVars(el, themeVars, mergedVars);
  }
}

export { OverrideStyleManager, assignThemeVars };
