import { defineComponent, ref, onMounted, onUnmounted, computed, watch, openBlock, createElementBlock, normalizeStyle, normalizeClass, unref, renderSlot } from 'vue';
import clx from 'clsx';
import { isEqual } from 'lodash';
import { resolveThemeMode, mediaQueryColorScheme, getAccent, getAccentText } from '../../helpers/style.js';
import { darkThemeClass, lightThemeClass } from '../../styles/themes.css.js';
import { isSSR } from '../../helpers/platform.js';
import { store } from '../../models/store.js';
import { assignThemeVars } from '../../styles/override/override.js';

const _hoisted_1 = ["data-is-controlled", "data-interchain-theme-mode"];
const _sfc_main = /* @__PURE__ */ defineComponent({
  __name: "theme-provider",
  props: {
    accent: {},
    defaultTheme: {},
    themeMode: {},
    overrides: {},
    children: {},
    themeDefs: {},
    customTheme: {},
    className: {},
    class: {},
    classList: {},
    forwardedRef: {}
  },
  setup(__props) {
    const props = __props;
    const preferredMode = ref(null);
    const isMounted = ref(false);
    ref(null);
    ref([]);
    const internalTheme = ref("light");
    const UIStore = ref(store.getState());
    const cleanupRef = ref();
    onMounted(() => {
      isMounted.value = true;
      const resolvedThemeMode = resolveThemeMode(props.defaultTheme);
      if (!isControlled) {
        store.getState().setThemeMode(resolvedThemeMode);
      }
      const darkListener = ({ matches }) => {
        if (matches) {
          preferredMode.value = "dark";
        }
      };
      const lightListener = ({ matches }) => {
        if (matches) {
          preferredMode.value = "light";
        }
      };
      const cleanupStore = store.subscribe((newState) => {
        UIStore.value = newState;
        internalTheme.value = newState.theme;
      });
      if (darkQuery && lightQuery) {
        if (typeof darkQuery.addEventListener === "function" && typeof lightQuery.addEventListener === "function") {
          darkQuery?.addEventListener("change", darkListener);
          lightQuery?.addEventListener("change", lightListener);
        }
      }
      cleanupRef.value = () => {
        if (typeof darkQuery.removeEventListener === "function") {
          darkQuery?.removeEventListener("change", darkListener);
        }
        if (typeof lightQuery.removeEventListener === "function") {
          lightQuery?.removeEventListener("change", lightListener);
        }
        if (typeof cleanupStore === "function") {
          cleanupStore();
        }
      };
    });
    onUnmounted(() => {
      if (typeof cleanupRef.value === "function") {
        cleanupRef.value();
      }
    });
    const isControlled = computed(() => {
      return props.themeMode != null;
    });
    const isReady = computed(() => {
      return preferredMode.value && isMounted.value;
    });
    const lightQuery = computed(() => {
      if (isSSR()) return null;
      return window?.matchMedia?.(mediaQueryColorScheme(`light`));
    });
    const darkQuery = computed(() => {
      if (isSSR()) return null;
      return window?.matchMedia?.(mediaQueryColorScheme(`dark`));
    });
    computed(() => {
      return !!darkQuery?.matches;
    });
    computed(() => {
      return !!lightQuery?.matches;
    });
    const themeClass = computed(() => {
      return getNewThemeClass(store.getState());
    });
    watch(
      () => [preferredMode.value, internalTheme.value, isReady, UIStore.value],
      () => {
        if (!isReady) {
          return;
        }
        const themeMode = store.getState().themeMode;
        const setThemeModeFn = store.getState().setThemeMode;
        if (themeMode === "system" || themeMode == null) {
          return setThemeModeFn("system");
        }
      },
      { immediate: true }
    );
    watch(
      () => [props.themeDefs, isControlled],
      () => {
        const finalThemeDefs = props.themeDefs ?? [];
        const isValidThemeDefs = Array.isArray(props.themeDefs) && finalThemeDefs.length > 0;
        if (!isValidThemeDefs) {
          return;
        }
        if (isControlled) {
          return;
        } else {
          if (!isEqual(store.getState().themeDefs, finalThemeDefs)) {
            store.getState().setThemeDefs(finalThemeDefs);
          }
        }
      },
      { immediate: true }
    );
    watch(
      () => [props.customTheme],
      () => {
        if (!props.customTheme) return;
        store.getState().setCustomTheme(props.customTheme);
      },
      { immediate: true }
    );
    watch(
      () => [props.overrides],
      () => {
        const overrideStyleManager = store.getState().overrideStyleManager;
        if (!overrideStyleManager) {
          return;
        }
        overrideStyleManager.update(props.overrides, null);
      },
      { immediate: true }
    );
    watch(
      () => [props.accent],
      () => {
        if (!props.accent) return;
        const prevAccent = UIStore.value.themeAccent;
        const currentColorMode = UIStore.value.theme;
        if (prevAccent !== props.accent) {
          UIStore.value.setThemeAccent(props.accent ?? "blue");
          assignThemeVars(
            {
              colors: {
                // @ts-ignore
                accent: getAccent(props.accent, currentColorMode ?? "light"),
                // @ts-ignore
                accentText: getAccentText(currentColorMode ?? "light")
              }
            },
            currentColorMode
          );
        }
      },
      { immediate: true }
    );
    function getNewThemeClass(uiStore) {
      if (isControlled) {
        if (props.themeMode === "system") {
          const finalThemeMode = isReady ? preferredMode.value : props.themeMode;
          return finalThemeMode === "dark" ? darkThemeClass : lightThemeClass;
        }
        return props.themeMode === "dark" ? darkThemeClass : lightThemeClass;
      }
      return uiStore.themeClass;
    }
    return (_ctx, _cache) => {
      return openBlock(), createElementBlock("div", {
        "data-is-controlled": isControlled.value,
        "data-interchain-theme-mode": _ctx.themeMode == null ? void 0 : _ctx.themeMode,
        style: normalizeStyle({
          visibility: isMounted.value ? "visible" : "hidden"
        }),
        class: normalizeClass(unref(clx)(themeClass.value, _ctx.className))
      }, [
        renderSlot(_ctx.$slots, "default")
      ], 14, _hoisted_1);
    };
  }
});

export { _sfc_main as default };
